/*
 *  libmSDL
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

#include "mSDL.hpp"


namespace mSDL
{

SDL_Surface* Screen = NULL;
KeyHandler Key;
MouseHandler Mouse;
//MouseHandler& Stylus = Mouse;


void Init( u16 LogicalScreenWidth, u16 LogicalScreenHeight )
{
    // initialize SDL with video, audio and timer modules
    if ( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER ) < 0 )
    {
        std::cerr << "mSDL::Init(): " << SDL_GetError() << std::endl;
        exit( 1 );
    }
    atexit( SDL_Quit );

    // initialize the screen surface with default settings
    Screen = SDL_SetVideoMode( DEFAULT_SCREEN_WIDTH, DEFAULT_SCREEN_HEIGHT,
                               DEFAULT_SCREEN_BPP, DEFAULT_SCREEN_FLAGS );
    if ( !Screen )
    {
        std::cerr << "mSDL::Init(): " << SDL_GetError() << std::endl;
        exit( 1 );
    }

    if( ( LogicalScreenWidth != DEFAULT_SCREEN_WIDTH ) ||
        ( LogicalScreenHeight != DEFAULT_SCREEN_HEIGHT ) )
    {
        // make Screen point to a custom surface instead of the real screen
        Screen = SDL_CreateSurface(LogicalScreenWidth, LogicalScreenHeight);
        /* REMOVE?
        SDL_Surface* t = SDL_CreateRGBSurface( SDL_HWSURFACE,
                                               LogicalScreenWidth,
                                               LogicalScreenHeight,
                                               Screen->format->BitsPerPixel,
                                               Screen->format->Rmask,
                                               Screen->format->Gmask,
                                               Screen->format->Bmask,
                                               Screen->format->Amask );
        if ( !t )
        {
            std::cerr << "mSDL::Init: " << SDL_GetError() << std::endl;
            exit( 1 );
        }
        Screen = SDL_DisplayFormat( t );
        */
        if ( !Screen )
        {
            std::cerr << "mSDL::Init: " << SDL_GetError() << std::endl;
            exit( 1 );
        }
    }

#ifdef _HAS_SDL_MIXER
    // initialize the audio device and SDL_Mixer
    if( Mix_OpenAudio( MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, MIX_DEFAULT_CHANNELS, MIX_CHUNKSIZE ) < 0 )
    {
        std::cerr << "mSDL::Init(): " << SDL_GetError() << std::endl;
        exit( 1 );
    }
    atexit( Mix_CloseAudio );

    // allocate channels for sounds mixing
    Mix_AllocateChannels( MIX_CHANNELS );

#endif

#ifdef _HAS_SDL_TTF
    // initialize the SDL_TTF module
    if( TTF_Init() < 0 )
    {
        std::cerr << "mSDL::Init(): " << SDL_GetError() << std::endl;
        exit( 1 );
    }
    atexit( TTF_Quit );
    // initialize the default font used by the Banner class
    //REMOVE? Banner::SetDefaultFont();
#endif

}


u8 GetFPS( void )
{
    // implemented using Last-Second FPS, code from http://www.libsdl.org/cgi/docwiki.cgi/SDL_Average_FPS_Measurement

    static u32 fps_lasttime = SDL_GetTicks(); //the last recorded time.
    static u32 fps_current; //the current FPS.
    static u32 fps_frames = 0; //frames passed since the last recorded fps.

    fps_frames++;

    if (fps_lasttime < SDL_GetTicks() - 1000)
    {
        fps_lasttime = SDL_GetTicks();
        fps_current = fps_frames;
        fps_frames = 0;
    }

    return fps_current;
}


}
