/*
 *  libmSDL
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

/*! \file mSDL_Banner.hpp
    \class Banner
    \brief Text banners handling module
*/

#ifdef _HAS_SDL_TTF

#include <SDL_ttf.h>


class Banner : public ScreenObject
{
    private:

        std::string Text;
        SDL_Color Color;
        u16 Size;
        std::string FontFilename;
        TTF_Font* Font;

        /* TODO:
        bool Bold;
        bool Underlined;
        bool Italic;
        */
        //TODO: bool Warp; //!< together with Move() this produces a marquee-like text banner
        //TODO: bool Wrap; -> manda in testo automaticamente a capo quando va fuori dallo schermo...

        void Render() // render the graphic surface containing the text banner. Is it invoked ONLY on text, font or color change.
        {
            if( Surface )
                SDL_FreeSurface( Surface );

            if( Text.empty() )
            {
                Surface = NULL; // Surface == NULL is no problem for Draw?
                return;
            }

            if( Font )
                TTF_CloseFont( Font );

            Font = TTF_OpenFont( FontFilename.c_str(), Size );
            if( ! Font )
            {
                std::cerr << "mSDL::Banner::Render(): " << TTF_GetError() << std::endl;
                exit( 1 );
            }

            Surface = TTF_RenderText_Blended( Font, Text.c_str(), Color );
            SetNoFrames( 1 );
        }

        //REMOVE? static TTF_Font* DefaultFont; //!< default font used by Banner class (inizialized by mSDL::Init())

    public:

        //REMOVE? static void SetDefaultFont( int ptsize = DEFAULT_FONT_SIZE, const char* file = DEFAULT_FONT_FILE ); //!< set the default font used by Banner class

        Banner( const std::string InitText = "",
                const SDL_Color InitColor = BLACK,
                const u16 InitSize = DEFAULT_FONT_SIZE,
                const std::string InitFontFilename = DEFAULT_FONT_FILE
              ) //!< banner constructor. Please do not close font while you are using it
        {

            Text = InitText;
            Color = InitColor;
            Size = InitSize;

            if( InitFontFilename.empty() )
                FontFilename = DEFAULT_FONT_FILE;
            else
                FontFilename = InitFontFilename;

            Font = NULL;

            Render();

            Prio = DEFAULT_BANNER_PRIO;
            Register();
        }

        ~Banner()
        {
            if( Font )
                TTF_CloseFont( Font );
        }

        /*  TODO: test!
        Banner( const Banner& orig ) // copy constructor
        {
            Render();
            Register();
        }

        Banner& operator=( const Banner& orig ) // assignment operator
        {
            if( this == &orig )
                return *this;

            Render();
        }
        */

        virtual void SetPrio( const u16 NewPrio = DEFAULT_BANNER_PRIO ) { ScreenObject::SetPrio( NewPrio ); }


        std::string GetText( void ) const { return Text; } //!< Return a copy of the object's text.
        void SetText( std::string NewText = "" ) //!< Change the banner text. If invoked with no arguments empty the banner.
        {
            Text = NewText;
            Render();
        }

        void SetFont( const std::string NewFontFilename = DEFAULT_FONT_FILE ) //!< Change the banner's fontface. Please avoid using absolute paths since they are usually not portable. If invoked with no arguments resets to default font.
        {
            FontFilename = NewFontFilename;
            Render();
        }

        u16 GetSize( void ) const { return Size; }
        void SetSize( const u16 NewSize = DEFAULT_FONT_SIZE )
        {
            Size = NewSize;
            Render();
        }

        /* REMOVE?
        TTF_Font* GetFont( void ) const { return Font; }
        void SetFont( TTF_Font* NewFont = DefaultFont )
        {
            if( !NewFont ) return;
            Font = NewFont;
            Render();
        }
        */

        SDL_Color GetColor( void ) const { return Color; }
        SDL_Color SetColor( const SDL_Color NewColor = BLACK )
        {
            Color = NewColor;
            Render();
        }

        virtual void Draw()
        {
            ScreenObject::Draw();

            // TODO: warping -> copiare codice da Sprite?
        }

        // TODO: animations
        // http://www.siforge.org/articles/2004/10/26-sdl_ttf.html

};
#endif
