/*
 *  libmSDL
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

/*! \file mSDL_Music.hpp
    \class Music
    \brief Music playback handling module

    MEMO: Only one Music istance can be playing at a time.
*/

#ifdef _HAS_SDL_MIXER

#include <SDL_mixer.h>

class Music
{
    private:
        Mix_Music* m;

    public:
        //! load a music file
        //! (WAVE, MOD, MIDI, OGG, MP3 supported)
        Music( const std::string Filename )
        {
            m = Mix_LoadMUS( Filename.c_str() );
            if( !m )
            {
                // loading error
                std::cerr << "mSDL::Music(...): " << SDL_GetError() << std::endl;
                return;
                //ALTERNATIVE BEHAVIOR: exit( 1 ); // abort execution
            }
        }

        ~Music()
        {
            if(m==NULL) return;
            Mix_FreeMusic(m);
        }

        void Play(int Loops=1)/*NO?, bool FadeIn = false, bool FadeOut = false)*/
        {
            if(m==NULL) return;
            Mix_PlayMusic(m, Loops);
        }

        static void Pause()
        {
            Mix_PauseMusic();
        }

        static void Resume()
        {
            Mix_ResumeMusic();
        }

        static void Stop()
        {
            Mix_HaltMusic();
        }

        //TODO? double GetPosition()
        static void SetPosition(double NewPosition) //! Set the position of the currently playing music
        {
            Mix_SetMusicPosition(NewPosition);
        }

        static int GetVolume()
        {
            return(Mix_VolumeMusic(-1));
        }

        static void SetVolume(int Volume = MIX_MAX_VOLUME)
        {
            Mix_VolumeMusic(Volume);
        }

        static void Mute()
        {
            Mix_VolumeMusic(0);
        }

        //TODO: void SetFadeIn(lenght);
        //TODO: void SetFadeOut(lenght);
};

#endif
