/*
 *  libmSDL
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

#include "mSDL.hpp"
#include "SDL_Utils.h"

namespace mSDL
{

ScreenObject::MultisetSortedByPrio ScreenObject::RegisteredScreenObjects;


void ScreenObject::Move( void )
{
    if( !isMoving() ) return;

    s32 PassedTimeSinceLastMove = SDL_GetTicks() - LastMoveTime;

    s16 NewX = GetX();
    s16 NewY = GetY();

    if( GetVX() )
    {
        s16 Step = PassedTimeSinceLastMove / ( 1000 / VX );
        if( Step != 0 )
        {
            NewX += Step;
            LastMoveTime = SDL_GetTicks();
        }
    }

    if( GetVY() )
    {
        s16 Step = PassedTimeSinceLastMove / ( 1000 / VY );
        if( Step != 0 )
        {
            NewY += Step;
            LastMoveTime = SDL_GetTicks();
        }
    }

    SetXY( NewX, NewY );

}



void ScreenObject::Draw( void )
{
    UpdateFrame();
    Move();

    SDL_BlitSurface( Surface, &GetFrameRect(), Screen, &GetPosition() );
}

/*TODO
void ScreenObject::ProcessAll( void (*CallbackFunction)(void) )
{
    if( RegisteredScreenObjects.empty() )
        return;

    MultisetSortedByPrio::const_iterator i;
    for( i=RegisteredScreenObjects.begin(); i!=RegisteredScreenObjects.end(); i++ )
        //(*i)->CallbackFunction()
        ;
}*/

void ScreenObject::DrawAll( void )
{
    if( !Screen || RegisteredScreenObjects.empty() )
        return;

    // frame-rate limiter code from:
    // http://listas.apesol.org/pipermail/sdl-libsdl.org/2001-May/017076.html
    u32 wait=(u32)(1000 / MAX_FPS);
    u32 cticks, bticks;
    bticks=SDL_GetTicks();

     // iterate throught registered screen objects and draw them
     // TODO: ProcessAll( &Draw );
    MultisetSortedByPrio::const_iterator i;
    for( i=RegisteredScreenObjects.begin(); i!=RegisteredScreenObjects.end(); i++ )
        if( (*i)->Visible && (*i)->Surface )
            (*i)->Draw();

    // checks if RealScreen != LogicalScreen and performs scaling if necessary
    SDL_Surface* LogicalScreen = Screen;
    SDL_Surface* RealScreen = SDL_GetVideoSurface();
    if( RealScreen != LogicalScreen )
    {
        SDL_ScaleSurface( LogicalScreen, RealScreen, RealScreen->w, RealScreen->h );
    }

    SDL_Flip( RealScreen );

    // checks if it is necessary to sleep for desired FPS
    cticks=SDL_GetTicks();
    if ( (cticks-bticks) < wait ) /* framerate exceeded the limit... so we wait the difference */
        SDL_Delay( wait-(cticks-bticks) );

}


}
