/*
 *  libmSDL
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

/*! \file mSDL_Sound.hpp
    \class Sound
    \brief Sound playback and mixing handling module
*/

#ifdef _HAS_SDL_MIXER

#include <SDL_mixer.h>
//#include "SDL_sfxr.h"


class Sound
{
    private:
        Mix_Chunk* c;

    public:
        //! load a sound file
        //! (WAVE, AIFF, RIFF, OGG, and VOC supported)
        Sound( const std::string Filename )
        {
            c = Mix_LoadWAV( Filename.c_str() );
            if( !c )
            {
                // loading error
                std::cerr << "mSDL::Sound(...): " << SDL_GetError() << std::endl;
                return;
                //ALTERNATIVE BEHAVIOR: exit( 1 ); // abort execution
            }
        }

/* TODO?
    //PSG - generate a sound via software
    //PALib-like: u8 pan, int freq, u8 duty
        Sound(int len, int PSG_preset)
        {
            //using namespace sfxr;
            //using mSDL::sfxr::getMix_Chunk;

            c = sfxr::getMix_Chunk(len, PSG_preset);
        }*/

        ~Sound()
        {
            if(c==NULL) return;
            Mix_FreeChunk(c);
        }

        void Play(int Loops=0) //! play this sound. Loops = -1 -> plays infinitely, Loops = 0 -> plays 1 time.
        //TO CHECK: what happens if there is no available channels?
        {
            if(c==NULL) return;
            Mix_PlayChannel(-1, c, Loops);
        }


        static void Pause() //! pause all samples playback
        {
            Mix_Pause(-1);
        }

        static void Resume() //! resume playback on all previously active channels
        {
            Mix_Resume(-1);
        }

        static void Stop() //! halt playback on all channels
        {
            Mix_HaltChannel(-1);
        }

        int GetVolume()
        {
            if(c==NULL) return(-1);
            return(Mix_VolumeChunk(c, -1));
        }

        void SetVolume(int Volume = MIX_MAX_VOLUME)
        {
            if(c==NULL) return;
            Mix_VolumeChunk(c, Volume);
        }

        void Mute()
        {
            if(c==NULL) return;
            Mix_VolumeChunk(c, 0);
        }

        //TODO: void SetFadeIn(lenght);
        //TODO: void SetFadeOut(lenght);
};

#endif
