/*
 *  libmSDL
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

#include "mSDL.hpp"

namespace mSDL
{


void Sprite::SetXY( const s16 NewX, const s16 NewY )
{
    ScreenObject::SetXY( NewX, NewY );

    if( Constrained )
    {
        if( ( NewX + W ) > (Screen->w) ) SetX( (Screen->w) - W );
        else if( NewX < 0 ) SetX( 0 );

        if( ( NewY + H ) > (Screen->h) ) SetY( (Screen->h) - H );
        else if( NewY < 0 ) SetY( 0 );
    }
    else if( Warp )
    {
        if( NewX > (Screen->w) ) SetX( NewX - (Screen->w) );
        else if( NewX < 0 ) SetX( Screen->w + NewX );

        if( NewY > (Screen->h) ) SetY( NewY - (Screen->h) );
        else if( NewY < 0 ) SetY( Screen->h + NewY );
    }
}


void Sprite::Draw() // draw a single sprite on the screen
{
    ScreenObject::Draw();

    if( !Warp ) // 2ADD: || isOnScreen() ?
        return;

    // else do multiple blittings
    SDL_Rect CurrentFrame = GetFrameRect();
    SDL_Rect ScreenPosition = GetPosition();

    // TODO: more optimized version?
    if( ( GetX() + W ) > ( Screen->w ) )
    {
        //ScreenPosition.x = 0;
        //CurrentFrame.w = (X+W) - (Screen->w);
        //CurrentFrame.x = W - CurrentFrame.w; // * Frame
        ScreenPosition.x = GetX() - (Screen->w);
        SDL_BlitSurface( Surface, &CurrentFrame, Screen, &ScreenPosition );
    }

    if( ( GetY() + H ) > ( Screen->h ) )
    {
        //ScreenPosition.y = 0;
        //CurrentFrame.h = (Y+H) - (Screen->h);
        //CurrentFrame.y = H - CurrentFrame.h; // * Frame
        ScreenPosition.x = GetX();
        ScreenPosition.y = GetY() - (Screen->h);
        SDL_BlitSurface( Surface, &CurrentFrame, Screen, &ScreenPosition );
    }

    if( ( ( GetX() + W ) > ( Screen->w ) ) &&
        ( ( GetY() + H ) > ( Screen->h ) ) )
    {
        /*
        ScreenPosition.x = X;
        ScreenPosition.y = 0;
        CurrentFrame.h = (Y+H) - (Screen->h);
        CurrentFrame.w = (X+W) - (Screen->w);
        CurrentFrame.x = W - CurrentFrame.w; // * Frame
        CurrentFrame.y = H - CurrentFrame.h; // * Frame
        */
        ScreenPosition.x = GetX() - (Screen->w);
        ScreenPosition.y = GetY() - (Screen->h);
        SDL_BlitSurface( Surface, &CurrentFrame, Screen, &ScreenPosition );
    }

}


void Sprite::HideAll() // static
{
    if( !Screen || RegisteredScreenObjects.empty() )
        return;

    MultisetSortedByPrio::const_iterator i;
    for( i=RegisteredScreenObjects.begin(); i!=RegisteredScreenObjects.end(); i++ )
        if( ( typeid( **i ).name() == typeid( Sprite ).name() ) && (*i)->isVisible() )
            (*i)->SetVisible(false);
}


void Sprite::ShowAll() // static
{
    if( !Screen || RegisteredScreenObjects.empty() )
        return;

    MultisetSortedByPrio::const_iterator i;
    for( i=RegisteredScreenObjects.begin(); i!=RegisteredScreenObjects.end(); i++ )
        if( ( typeid( **i ).name() == typeid( Sprite ).name() ) && !((*i)->isVisible() ) )
            (*i)->SetVisible(true);
}

/*
void Sprite::DrawAll() // static
{
    if( !Screen || RegisteredScreenObjects.empty() )
        return;

    MultisetSortedByPrio::const_iterator i;
    for( i=RegisteredScreenObjects.begin(); i!=RegisteredScreenObjects.end(); i++ )
        if( ( typeid( **i ).name() == typeid( Sprite ).name() ) && (*i)->Visible )
            (*i)->Draw();
}
*/

}
