/*
 *  libmSDL
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

/*! \file mSDL_Sprite.hpp
    \class Sprite
    \brief Sprite objects handling module
*/

class Sprite : public ScreenObject
{
    private:

        bool Warp; //!< true if the sprite is warped when collide with the edges of the screen
        bool Constrained; //!< true if sprite can't go out of the screen (even partially)

        void Init()
        {
            Warp = false;
            Constrained = false;

            Prio = DEFAULT_SPRITE_PRIO;
            Register();
        }


    public:

        //! Sprite constructor
        Sprite()
        {
            Init();
        }

        //REMOVE?
        using GFX::operator=; // not inherited by default

        //! Alternative sprite constructor
        Sprite( const GFX& InitGFX )
        {
            Init();
            //*this = InitGFX;
            GFX::operator=( InitGFX );
        }

        bool IsWarped() { return(Warp); }
        void SetWarped( bool w = true ) { Warp = w; }

        bool IsConstrained() { return(Constrained); }
        void SetConstrained( bool c = true ) { Constrained = c; }

        virtual void SetPrio( u16 NewPrio = DEFAULT_SPRITE_PRIO ) { ScreenObject::SetPrio( NewPrio ); }
        virtual void SetXY( s16 NewX = 0, s16 NewY = 0 ); // overloading ScreenObject::SetXY
        virtual void Draw(); //!< draw a single sprite on the screen. Usually this in not invoked directly (except you dont wanna use default DrawScreen and/or want to override the priority system).
        //static void DrawAll(); //!< draw ONLY and ALL the registered sprites (NO BGs, NO Banners). Normally it is not invoked by mSDL. Meant to be used in conjuction with custom BG drawing routines. Note that by using this you won't have to invoke WaitForVBL().

        static void HideAll(); //!< hide all the sprites (but do not unregister 'em all)
        static void ShowAll(); //!< set all the sprites visible

};


inline void HideAllSprites() { Sprite::HideAll(); }
inline void ShowAllSprites() { Sprite::ShowAll(); }
//inline void DrawAllSprites() { Sprite::DrawAll(); }
