/*
 *  Copyright 2007, 2008, 2009 eadmaster <spamhereplease@fastwebnet.it>
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

// portable (and fast) math funcitons


/* Fast math using bitwise operators - only for ints! */
inline int    square(int x) { return( x * x ); } // x^2
inline int      cube(int x) { return( x * x * x); } // x^3
inline int      half(int x) { return( x >> 1); } // division by 2
inline int   quarter(int x) { return( x >> 2); } // division by 4
inline int     doubl(int x) { return( x << 1); } // multiply by 2
inline int quadruple(int x) { return( x << 2); } // multiply by 4
/*
#define isodd(x) (x&1)
#define iseven(x) !(isodd(x))
#define max(x, y) ((x) > (y) ? (x) : (y) )
#define min(x, y) ((x) < (y) ? (x) : (y) )
*/


#include <math.h> // for sqrt
inline int isqrt(int x) // fast integer square root
{
    #if defined( NDS )
        return( swiSqrt( x ) );
    #else
        return( sqrt( x ) );

        // TODO: choose an algorithm from:
        // http://guru.multimedia.cx/fast-integer-square-root/
        // http://www.azillionmonkeys.com/qed/sqroot.html
        // http://www.embedded.com/98/9802fe2.htm
        // http://medialab.freaknet.org/martin/src/sqrt/
    #endif
}

//inline int rad2deg( double radians ); // convert radians to degrees
inline int pmath_2PointsAngle(int x1, int y1, int x2, int y2) { return( atan2( y2 - y1, x2 - x1 ) * 180 / M_PI ); } // Compute the angle (in degrees) between two points: P1( x1, x2 ) and P2( x2, y2 )
inline int pmath_2PointsDistance(int x1, int y1, int x2, int y2) { return( isqrt( square( x1 - x2 ) + square( y1 - y2 ) ) ); } // Compute the distance between two points: P1( x1, x2 ) and P2( x2, y2 )

inline bool pmath_collisionrect(int x1, int y1, int w1, int h1, int x2, int y2, int w2, int h2, int range)
{
    // code from http://www.palib.info/wiki/doku.php?id=day6#rectangular_sprites
    // TODO: aggiungere "range":
    return( ( x2 >= x1 - half( w1 + w2 ) ) &&
            ( x2 <= x1 + half( w1 + w2 ) ) &&
            ( y2 >= y1 - half( h1 + h2 ) ) &&
            ( y2 <= y1 + half( h1 + h2 ) ) );
}

inline bool pmath_collisioncirc(int x1, int y1, int r1, int x2, int y2, int r2, int range)
{
    // code from http://www.palib.info/wiki/doku.php?id=day6#round_sprites
    return( pmath_2PointsDistance( x1, y1, x2, y2 ) < ( r1 + r2 + range ) );
}
