// This file works both as a test and as a demonstration of basic mSDL features

#include <iostream.h>
using namespace std;

#include "libmsdl/mSDL.hpp"
using namespace mSDL;

#include <SDL_image.h>

#include <sstream>


int main ( int argc, char** argv )
{
    // Step 0: initialization
    mSDL::Init();

    Mouse.HideCursor();


    // Step 1: graphic objects creation
    // in this step we load graphic files into "GFX" objects
        GFX tiledgfx("bgtiled.bmp");

        GFX logogfx(IMG_Load("logo.png"));
        logogfx.SetTrans(BLACK); // set the transparent color
        // ALTERNATIVE: GFX logogfx( "logo.png", BLACK ); // black is the transparent color

        GFX tankgfx(IMG_Load("BipedalTank-Mjkrzak.png"));
        tankgfx.SetTrans(WHITE);
        tankgfx.SetNoFrames(7);
        // ALTERNATIVE: GFX tankgfx( "BipedalTank-Mjkrzak.png", WHITE, 7 ); // this is a spritesheet with 7 frames

        GFX test(YELLOW); // create an uniform color GFX
        GFX bg0gfx = test.Clone(); // test Clone method
        GFX bg1gfx(IMG_Load("bg1.png"), BLACK);


    // Step 2: create some screen objects

        // first we declare an empty sprite:
        Sprite logo;
        // since it is declared in the stack it will stay both in memory
        // and on screen until the end of this function's scope

        // by default it will stay invisible until we assign
        // a graphic surface with the assignment operator "="
        logo = logogfx;
        // with this you assign not only the graphic surface,
        // but also the "framing state" of the GFX object

        // or, shortly, you can use the alternative Sprite constructor
        // provided as a shortcut for the 2 above istructions:
        // Sprite logo( logogfx );

        // Now that the sprite is created,
        // we can use a lot of methods to modify the sprite's state.
        // For istance, we can change its position and priority:
        logo.SetX(200);
        logo.SetY(200);
        logo.SetPrio(10);

        // You can make a screen object invisible without deleting it
        // with the hide/show methods:
        logo.Hide();
        logo.Show();

        // Now we declare a sprite using another tecnique: dynamic memory.
        // It has the advantage that you can delete the sprite whenever you want
        // with the "delete" operator.
        Sprite* tank = new Sprite(tankgfx);

        // GFX objects with multiple frames can be animated
        tank->StartAnim();
        //tank->SetFPS(5);


    // step 3: now we create some BackGrounds
        BG bg0(bg0gfx);
        bg0.SetPrio(102);

        BG* bg1 = new BG(bg1gfx);
        bg1->SetPrio(10);
        bg1->SetAlpha(128);

        BG* tiled = new BG(tiledgfx);
        tiled->SetTiled(); // tile the background to fill the screen
        tiled->SetPrio( 200 );

#ifdef _HAS_SDL_TTF
    // Step 4: create some text banners
        Banner* title = new Banner("ciao", RED);
        title->SetXY(100,100);
        title->SetPrio(20);
        title->SetText("Banner test"); // change the text of the banner
        title->SetColor(YELLOW);

        Banner istructions("mouse move tank, arrow keys move logo, q or ESC exit, other keys activate tests...");
        istructions.SetSize(18);
        istructions.SetColor(MAGENTA);

        Banner* fps = new Banner("0", WHITE);
        fps->SetXY(500,10);
        fps->SetPrio(0);

        Banner* collision = new Banner("Collision detected", GREEN);
        collision->SetXY(10,400);
        collision->SetVisible(false); // it will start invisible and will became visible if a collision is detected
#endif

#ifdef _HAS_SDL_MIXER
    // Step 5: let's play some music!
        Music bgm("FNKYTOWN.MOD");
        bgm.Play();

    // Step 6: load some sounds..
        Sound crash("crash.wav"); // load a sound

#endif


    // main loop
    while( 1 )
    {
        // check user input
        // move logo's sprite using arrow keys
        logo.SetVX( 100 * ( Key.Right.Held - Key.Left.Held ) );
        logo.SetVY( 100 * ( Key.Down.Held - Key.Up.Held ) );

        if( tank )
        {
            tank->SetXY( Mouse.X, Mouse.Y ); // move tank sprite following mouse

            if( CollisionRect( logo, *tank ) ) // check collision between tank and logo and makes the banner visible or not
            {
                //cout << argv[0] << ": collision detected" << endl;
#ifdef _HAS_SDL_TTF
                collision->SetVisible(true);
#endif
#ifdef _HAS_SDL_MIXER
                //crash.Play();
#endif
            }
        }

        if( Key.Q.Held || Key.Esc.Held )
            break; // exit main loop

        // check some keys to perform some tests...
            if( Mouse.Left.Newpress && tank )
            {
                cout << argv[0] << ": Sprite SetFrame test" << endl;
                tank->SetFrame( !( tank->GetFrame() ) );
            }
            if( Key.D.Newpress )
            {
                cout << argv[0] << ": Objects delete test" << endl;
                #ifdef _HAS_SDL_TTF
                if(title) { delete title; title = NULL; }
                collision->SetVisible(false);
                #endif
                if(tank) { delete tank; tank = NULL; }
                if(tiled) { delete tiled; tiled = NULL; }
            }
            if( Key.C.Newpress )
            {
                cout << argv[0] << ": Sprite Constrained test" << endl;
                logo.SetConstrained( !logo.IsConstrained() );
            }
            if( Key.W.Newpress )
            {
                cout << argv[0] << ": Sprite Warp test" << endl;
                logo.SetWarped( !logo.IsWarped() );
            }
            if( Key.H.Newpress )
            {
                cout << argv[0] << ": Sprite HideAll test" << endl;
                HideAllSprites();
            }
            if( Key.S.Newpress )
            {
                cout << argv[0] << ": BG scrolling test" << endl;
                tiled->SetPrio( 101 );
                tiled->SetVY( 100 * ( ! tiled->GetVY() ) );
                tiled->SetVX( 100 * ( ! tiled->GetVX() ) );
            }
        // END of tests


        WaitForVBL(); // draw the screen, update "Mouse" and "Key" object and also limit FPS

#ifdef _HAS_SDL_TTF
        // update the FPS banner
            stringstream out;
            out << (int) GetFPS();
            fps->SetText( out.str() );
#endif

    } // END OF main loop

    printf("Exited cleanly from mSDL test\n");

    return 0;
}
